#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2018, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.


import gi

gi.require_version("Tracker", "3.0")
from gi.repository import Gio, GLib
from gi.repository import Tracker

import logging

import trackertestutils.mainloop

import configuration


log = logging.getLogger(__name__)


class WakeupCycleTimeoutException(RuntimeError):
    pass


DEFAULT_TIMEOUT = 10


class MinerFsHelper:

    MINERFS_BUSNAME = "org.freedesktop.Tracker3.Miner.Files"
    MINERFS_OBJ_PATH = "/org/freedesktop/Tracker3/Miner/Files"
    MINER_IFACE = "org.freedesktop.Tracker3.Miner"
    MINERFS_CONTROL_BUSNAME = "org.freedesktop.Tracker3.Miner.Files.Control"
    MINERFS_INDEX_OBJ_PATH = "/org/freedesktop/Tracker3/Miner/Files/Index"
    MINER_INDEX_IFACE = "org.freedesktop.Tracker3.Miner.Files.Index"

    def __init__(self, dbus_connection):
        self.log = logging.getLogger(__name__)

        self.bus = dbus_connection

        self.loop = trackertestutils.mainloop.MainLoop()

        self.miner_fs = Gio.DBusProxy.new_sync(
            self.bus,
            Gio.DBusProxyFlags.DO_NOT_AUTO_START_AT_CONSTRUCTION,
            None,
            self.MINERFS_BUSNAME,
            self.MINERFS_OBJ_PATH,
            self.MINER_IFACE,
        )

        self.index = Gio.DBusProxy.new_sync(
            self.bus,
            Gio.DBusProxyFlags.DO_NOT_AUTO_START_AT_CONSTRUCTION,
            None,
            self.MINERFS_CONTROL_BUSNAME,
            self.MINERFS_INDEX_OBJ_PATH,
            self.MINER_INDEX_IFACE,
        )

    def start(self):
        self.miner_fs.Start()

    def stop(self):
        self.miner_fs.Stop()

    def get_sparql_connection(self):
        return Tracker.SparqlConnection.bus_new(
            "org.freedesktop.Tracker3.Miner.Files", None, self.bus
        )

    def start_watching_progress(self):
        self._previous_status = None
        self._target_wakeup_count = None
        self._wakeup_count = 0

        def signal_handler(proxy, sender_name, signal_name, parameters):
            if signal_name == "Progress":
                self._progress_cb(*parameters.unpack())

        self._progress_handler_id = self.miner_fs.connect("g-signal", signal_handler)

    def stop_watching_progress(self):
        if self._progress_handler_id != 0:
            self.miner_fs.disconnect(self._progress_handler_id)

    def _progress_cb(self, status, progress, remaining_time):
        if self._previous_status is None:
            self._previous_status = status
        if self._previous_status != "Idle" and status == "Idle":
            self._wakeup_count += 1

        if (
            self._target_wakeup_count is not None
            and self._wakeup_count >= self._target_wakeup_count
        ):
            self.loop.quit()

    def wakeup_count(self):
        """Return the number of wakeup-to-idle cycles the miner-fs completed."""
        return self._wakeup_count

    def await_wakeup_count(self, target_wakeup_count, timeout=DEFAULT_TIMEOUT):
        """Block until the miner has completed N wakeup-and-idle cycles.

        This function is for use by miner-fs tests that should trigger an
        operation in the miner, but which do not cause a new resource to be
        inserted. These tests can instead wait for the status to change from
        Idle to Processing... and then back to Idle.

        The miner may change its status any number of times, but you can use
        this function reliably as follows:

            wakeup_count = miner_fs.wakeup_count()
            # Trigger a miner-fs operation somehow ...
            miner_fs.await_wakeup_count(wakeup_count + 1)
            # The miner has probably finished processing the operation now.

        If the timeout is reached before enough wakeup cycles complete, an
        exception will be raised.

        """

        assert self._target_wakeup_count is None

        if self._wakeup_count >= target_wakeup_count:
            log.debug(
                "miner-fs wakeup count is at %s (target is %s). No need to wait",
                self._wakeup_count,
                target_wakeup_count,
            )
        else:

            def _timeout_cb():
                raise WakeupCycleTimeoutException()

            timeout_id = GLib.timeout_add_seconds(timeout, _timeout_cb)

            log.debug(
                "Waiting for miner-fs wakeup count of %s (currently %s)",
                target_wakeup_count,
                self._wakeup_count,
            )
            self._target_wakeup_count = target_wakeup_count
            self.loop.run_checked()

            self._target_wakeup_count = None
            GLib.source_remove(timeout_id)

    def index_location(self, uri, graphs=None, flags=None):
        return self.index.IndexLocation("(sasas)", uri, graphs or [], flags or [])



# -*- coding: utf-8 -*-

# Copyright (C) 2010, Nokia (ivan.frade@nokia.com)
# Copyright (C) 2019-2020, Sam Thursfield (sam@afuera.me.uk)
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA.

import locale
import os
import unittest as ut

import configuration as cfg
import fixtures


class MinerFTSStopwordsTest(fixtures.TrackerMinerFTSTest):
    """
    Stop words were removed from Tracker SPARQL in 3.6

    See: https://gitlab.gnome.org/GNOME/tracker/-/merge_requests/611
    """

    def test_01_stopwords(self):
        """Test that you can search for English stopwords."""

        TEXT = "The optimism of the action is better than the pessimism of the thought"

        self.set_text(TEXT)

        results = self.search_word("the")
        self.assertEqual(len(results), 1)

        results = self.search_word("of")
        self.assertEqual(len(results), 1)

        results = self.search_word("in")
        self.assertEqual(len(results), 0)


if __name__ == "__main__":
    fixtures.tracker_test_main()



#!/usr/bin/env python3
# Copyright (C) 2020, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

"""Example script to query all documents from the filesystem index.

This uses an asynchronous query, which runs the query in a new thread and
notifies the main loop when results are ready.

"""

import gi
gi.require_version('Tracker', '3.0')
from gi.repository import Tracker, GLib


def results_ready_cb(obj, result, user_data):
    cursor = obj.query_finish(result)
    loop = user_data

    # This can also be done asynchronously using next_async().
    print("Documents:")
    while (cursor.next(None)):
        location = cursor.get_string(0)[0]
        title = cursor.get_string(1)[0]
        print(f"  * {location} (title: {title})")
    loop.quit ()


if __name__ == "__main__":
    loop = GLib.MainLoop ()
    cancellable = None

    miner_fs = Tracker.SparqlConnection.bus_new(
        "org.freedesktop.Tracker3.Miner.Files", None, cancellable)
    cursor = miner_fs.query_async(
        "SELECT nie:isStoredAs(?doc) ?title "
        "FROM tracker:Documents "
        "WHERE { ?doc a nfo:Document ; nie:title ?title . }",
        cancellable,
        results_ready_cb,
        loop)

    loop.run ()



# Copyright (C) 2023, Red Hat Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Tests misbehavior of tracker-extract modules.
"""

import os
import shutil
import sys
import unittest as ut

import configuration as cfg
import fixtures

RULES_DIR = ""
EXTRACTOR_DIR = ""

class MisbehaviorTest(fixtures.TrackerMinerTest):
    """
    Tests crawling and monitoring of configured content locations.
    """

    def environment(self):
        extra_env = cfg.test_environment(self.workdir)
        extra_env["TRACKER_EXTRACTOR_RULES_DIR"] = RULES_DIR
        extra_env["TRACKER_EXTRACTORS_DIR"] = EXTRACTOR_DIR
        return extra_env

    def setUp(self):
        monitored_files = self.create_test_data()

        try:
            # Start the miner.
            fixtures.TrackerMinerTest.setUp(self)

            for tf in monitored_files:
                url = self.uri(tf)
                self.tracker.ensure_resource(
                    fixtures.FILESYSTEM_GRAPH,
                    f"a nfo:FileDataObject; nie:url '{url}' ; tracker:extractorHash ?hash",
                    timeout=cfg.AWAIT_TIMEOUT,
                )
        except Exception:
            cfg.remove_monitored_test_dir(self.workdir)
            raise

    def create_test_data(self):
        monitored_files = [
            "test-monitored/file1.txt",
        ]

        for tf in monitored_files:
            testfile = self.path(tf)
            os.makedirs(os.path.dirname(testfile), exist_ok=True)
            with open(testfile, "w") as f:
                f.write('Some text')

        return monitored_files

    def test_misbehavior(self):
        self.assertEqual(self.tracker.query('ASK { <fail://> a rdfs:Resource }'), [['false']]);

if __name__ == "__main__":
    if len(sys.argv) < 3:
        sys.stderr.write("ERROR: missing rules dir and extractor module path arguments")
        sys.exit(1)

    RULES_DIR = sys.argv.pop(1)
    EXTRACTOR_DIR = sys.argv.pop(1)
    fixtures.tracker_test_main()



#!/usr/bin/env python3
# Tracker website build script.
#
# Copyright 2020, Sam Thursfield <sam@afuera.me.uk>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA.


import bs4

import argparse
import contextlib
import logging
import pathlib
import shutil
import subprocess
import sys
import tempfile

log = logging.getLogger('build.py')

website_root = pathlib.Path(__file__).parent
docs_root = website_root.parent
source_root = docs_root.parent

asciidoc = shutil.which('asciidoc')
mkdocs = shutil.which('mkdocs')
xmlto = shutil.which('xmlto')


def argument_parser():
    parser = argparse.ArgumentParser(
        description="Tracker website build script")
    parser.add_argument('--output', required=True, metavar='OUTPUT',
                        help="Output directory")
    parser.add_argument('--debug', dest='debug', action='store_true',
                        help="Enable detailed logging to stderr")
    parser.add_argument('--man-pages', nargs='+', required=True,
                        help="List of Asciidoc manual page source")
    return parser


class Manpages():
    def run(self, command):
        command = [str(c) for c in command]
        log.debug("Running: %s", ' '.join(command))
        subprocess.run(command, check=True)

    def generate_manpage_xml(self, in_path, out_path):
        """Generate a docbook XML file for an Asciidoc manpage source file."""
        self.run([asciidoc, '--backend', 'docbook', '--doctype', 'manpage',
                 '--out-file', out_path, in_path])

    def generate_manpage_html(self, in_path, out_path):
        """Generate a HTML page from a docbook XML file"""
        self.run([xmlto, 'xhtml-nochunks', '-o', out_path, in_path])

    def generate_toplevel_markdown(self, in_path, out_path, html_files):
        """Generate the master commandline.md page."""

        includes = []
        for path in sorted(html_files):
            parser = bs4.BeautifulSoup(path.read_text(), 'html.parser')

            title = parser.title.text
            if title and title.startswith('tracker3-'):
                title = 'tracker3 ' + title[9:]

            body = parser.body.contents[0]

            includes.append("## %s\n\n%s\n---\n" % (title, body))

        text = in_path.read_text()
        text = text.format(
            includes='\n'.join(includes)
        )
        out_path.write_text(text)


@contextlib.contextmanager
def tmpdir():
    path = pathlib.Path(tempfile.mkdtemp())
    log.debug("Created temporary directory %s", path)
    try:
        yield path
    finally:
        log.debug("Removed temporary directory %s", path)
        shutil.rmtree(path)


def main():
    args = argument_parser().parse_args()
    output_path = pathlib.Path(args.output)

    if args.debug:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    else:
        logging.basicConfig(stream=sys.stderr, level=logging.INFO)

    if output_path.exists():
        raise RuntimeError(f"Output path '{output_path}' already exists.")

    if asciidoc is None:
        raise RuntimeError("The 'asciidoc' tool is required.")
    if xmlto is None:
        raise RuntimeError("The 'xmlto' tool is required.")

    log.info("Generating online man pages")
    with tmpdir() as workdir:
        manpages = Manpages()

        htmldir = website_root.joinpath('docs')
        htmlfiles = []

        for man_page in args.man_pages:
            asciidocpath = pathlib.Path(man_page)
            xmlpath = workdir.joinpath(asciidocpath.stem + '.xml')

            manpages.generate_manpage_xml(asciidocpath, xmlpath)
            manpages.generate_manpage_html(xmlpath, htmldir)

            htmlpath = htmldir.joinpath(xmlpath.with_suffix('.html').name)
            htmlfiles.append(htmlpath)

        template_in = website_root.joinpath('commandline.md.in')
        template_out = website_root.joinpath('commandline.md')

        manpages.generate_toplevel_markdown(template_in, template_out, htmlfiles)


    log.info("Building website")
    mkdocs_config = docs_root.joinpath('mkdocs.yml')
    subprocess.run([mkdocs, 'build', '--config-file', mkdocs_config,
                    '--site-dir', output_path.absolute()],
                   check=True)

    log.info("Documentation available in %s/ directory.", args.output)


try:
    main()
except (RuntimeError, subprocess.CalledProcessError) as e:
    sys.stderr.write(f"ERROR: {e}\n")
    sys.exit(1)
